/* This file was generated by plugin '<%= PluginInfo.Name %>' (BDS version <%= PluginInfo.AppVersion %>) */

#include "ble_<%= ShortName %>.h"
#include <string.h>
#include "nordic_common.h"
#include "ble_srv_common.h"
#include "app_util.h"
#include "app_util_bds.h"

#define OPCODE_LENGTH 1 /**< Length of opcode inside <%= Name %> packet. */
#define HANDLE_LENGTH 2 /**< Length of handle inside <%= Name %> packet. */

/* TODO Consider changing the max values if encoded data for characteristic/descriptor is fixed length */ <%
_.each(Characteristics, function(characteristic) {
    if (characteristic.isReadSupported() || characteristic.isNotifySupported() || characteristic.isIndicateSupported() ||
        characteristic.isWriteSupported() || characteristic.isWriteWithoutResponseSupported() || characteristic.isReliableWriteSupported()) { %>
#define MAX_<%= characteristic.NormalizedName().toUpperCase() %>_LEN (BLE_L2CAP_MTU_DEF - OPCODE_LENGTH - HANDLE_LENGTH) /**< Maximum size of a transmitted <%= characteristic.Name %>. */ <%
    _.each(characteristic.Descriptors, function(descriptor) {
        if(descriptor.UUID === '2902') { // CCCD, this is already handled in SDK code
            return;
        }
        if(descriptor.isReadSupported() || descriptor.isWriteSupported()) { %>
#define MAX_<%= descriptor.NormalizedName().toUpperCase() %>_LEN (BLE_L2CAP_MTU_DEF - OPCODE_LENGTH - HANDLE_LENGTH) /**< Maximum size of a transmitted <%= descriptor.Name %>. */ <%
        }
    });
}}); %>
<%
var addedDescriptorUuids = [];

_.each(Characteristics, function(characteristic) {
    if (characteristic.isReadSupported() || characteristic.isNotifySupported() || characteristic.isIndicateSupported() ||
            characteristic.isWriteSupported() || characteristic.isWriteWithoutResponseSupported() || characteristic.isReliableWriteSupported()) {
    _.each(characteristic.Fields, function(field) {
        if (field.isStruct()) {
%>
/**@brief Function for encoding <%= field.Name %>.
 *
 * @param[in]   p_<%= field.NormalizedName() %>              <%= field.Name %> structure to be encoded.
 * @param[out]  p_encoded_buffer   Buffer where the encoded data will be written.
 *
 * @return      Size of encoded data.
 */
static uint8_t <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_encode(<%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_t * p_<%= field.NormalizedName() %>, uint8_t * encoded_buffer)
{
    <%= field.FormatType() %>_t <%= field.NormalizedName() %>;<%
        if (field.BitField && field.BitField.Bits.length > 0) {
            _.each(field.BitField.Bits, function(bitField) { %>
    <%= field.NormalizedName() %> = <%= field.NormalizedName() %> | (p_<%= field.NormalizedName() %>-><%= bitField.NormalizedName() %> << <%= bitField.Index %>); <%
            });
        } else { %>
    <%= field.NormalizedName() %> = p_<%= field.NormalizedName() %>-><%= field.NormalizedName() %>;<%
        }

        if (field.FormatType() === 'uint8' || field.FormatType() === 'int8' || field.FormatType() === 'nibble') { %>
    encoded_buffer[0] = <%= field.NormalizedName() %>;
    return 1; <%
    } else { %>

    return bds_<%= field.FormatType() %>_encode(&<%= field.NormalizedName() %>, &encoded_buffer[0]); <%
    } %>
}
<%
        }
    });

%>
/**@brief Function for encoding <%= characteristic.Name %>.
 *
 * @param[in]   p_<%= characteristic.NormalizedName() %>              <%= characteristic.Name %> characteristic structure to be encoded.
 * @param[out]  p_encoded_buffer   Buffer where the encoded data will be written.
 *
 * @return      Size of encoded data.
 */
static uint8_t <%= characteristic.NormalizedName() %>_encode(ble_<%= ShortName %>_<%= characteristic.NormalizedName() %>_t * p_<%= characteristic.NormalizedName() %>, uint8_t * encoded_buffer)
{
    uint8_t len = 0; <%

    if (characteristic.hasNibbles()) { %>
    uint8_t nibble = 0; <%
    }

    _.each(characteristic.Fields, function(field) {
    if(field.BitField) {
        _.each(field.BitField.Bits, function(bitField) {
            _.each(bitField.getEnumerations(), function(enumVal) {
                if (enumVal.hasRequires()) { %>
    bool <%= enumVal.Requires %>_required = (p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>.<%= bitField.NormalizedName() %> == <%= bitField.NormalizedName().toUpperCase() %>_<%= enumVal.NormalizedName().toUpperCase() %>); <%
                }
            });
        });
    }
    _.each(field.getEnumerations(), function(enumVal) {
        if (enumVal.hasRequires()) { %>
    bool <%= enumVal.Requires %>_required = (p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>.<%= field.NormalizedName() %> == <%= field.NormalizedName().toUpperCase() %>_<%= enumVal.NormalizedName().toUpperCase() %>); <%
            }
        });
    });

    var nibble = false;
    _.each(characteristic.Fields, function(field) {
        if(field.hasRequirement()) { %>
    if (<% for(var i=0; i<field.Requirement.length; i++) { if(i>0) { print(' && '); }%><%= field.Requirement[i] %>_required<%}%>)
    {<%
            if (field.isStruct()) {
                if (field.FormatType() === 'nibble') {
                    if (nibble == false) { %>
        <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>, &nibble);<%
                        nibble = true;
                    } else { %>
        uint8_t nib;
        <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>, &nib);
        nibble = nibble | nib << 4;
        encoded_buffer[len++] = nibble;
        nibble = 0;<%
                    nibble = false;
                    }
                   } else { %>
        len += <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>, &encoded_buffer[len]); <%
                 }
            } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8') { %>
        encoded_buffer[len++] = p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>;<%
            } else if (field.FormatType() === 'nibble') {
                if (nibble == false) { %>
        nibble = p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>;<%
                    nibble = true;
                   } else { %>
        nibble = nibble | (p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %> << 4);
        encoded_buffer[len++] = nibble;
        nibble = 0;<%
                    nibble = false;
                   }
            } else { %>
        len += bds_<%= field.FormatType() %>_encode(&p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>, &encoded_buffer[len]); <%
            } %>
    } <%

        } else {
            if (field.isStruct()) {
                if (field.FormatType() === 'nibble') {
                    if (nibble == false) { %>
    <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>, &nibble);<%
                        nibble = true;
                    } else { %>
    uint8_t nib;
    <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>, &nib);
    nibble = nibble | nib << 4;
    encoded_buffer[len++] = nibble;
    nibble = 0;<%
                        nibble = false;
                    }
                } else { %>
    len += <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>, &encoded_buffer[len]); <%
                }
            } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8') { %>
    encoded_buffer[len++] = p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>;<%
            } else if (field.FormatType() === 'nibble') {
                if (nibble == false) { %>
    nibble = p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>;<%
                    nibble = true;
                } else { %>
    nibble = nibble | (p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %> << 4);
    encoded_buffer[len++] = nibble;
    nibble = 0;<%
                    nibble = false;
                }
            } else { %>
    len += bds_<%= field.FormatType() %>_encode(&p_<%= characteristic.NormalizedName() %>-><%= field.NormalizedName() %>, &encoded_buffer[len]); <%
            }
        }
    });
%>
    return len;
}
<%
    }

    if (characteristic.isWriteSupported() || characteristic.isWriteWithoutResponseSupported() || characteristic.isReliableWriteSupported()) {
     _.each(characteristic.Fields, function(field) {
             if (field.isStruct()) {
     %>
/**@brief Function for decoding <%= field.Name %>.
 *
 * @param[in]   data_len              Length of the field to be decoded.
 * @param[in]   p_data                Buffer where the encoded data is stored.
 * @param[out]  p_write_val           Decoded data.
 *
 * @return      Length of the decoded field.
 */
 static uint8_t <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_decode(uint8_t data_len, uint8_t * p_data, <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_t * p_write_val)
 {
    uint8_t pos = 0; <%

        if (field.BitField && field.BitField.Bits.length > 0) {
            _.each(field.BitField.Bits, function(bitField) { %>
    p_write_val-><%= bitField.NormalizedName() %> = (enum_<%= field.NormalizedName() %>_<%= bitField.NormalizedName() %>_t)(p_data[pos] & (0x01 <<  <%= bitField.Index %>));<%
            }); %>
    pos += sizeof(<%= field.FormatType() %>_t);<%
        } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8' || field.FormatType() === 'nibble') { %>
    p_write_val-><%= field.NormalizedName() %> = <% if (field.isStruct()) { %>(enum_<%= field.NormalizedName() %>_t)<% }%>p_data[pos++]; <%
        } else { %>
    pos += bds_<%= field.FormatType() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
        } %>
    return pos;
 }
     <%
             }
         });
    %>
/**@brief Function for decoding <%= characteristic.Name %>.
 *
 * @param[in]   data_len              Length of the field to be decoded.
 * @param[in]   p_data                Buffer where the encoded data is stored.
 * @param[out]  p_write_val           Decoded data.
 *
 * @return      Length of the decoded field.
 */
static uint8_t <%= characteristic.NormalizedName() %>_decode(uint8_t data_len, uint8_t * p_data, ble_<%= ShortName %>_<%= characteristic.NormalizedName() %>_t * p_write_val)
{
    uint8_t pos = 0;<%

    if (characteristic.hasNibbles()) { %>
    uint8_t nibble = 0; <%
    }
     _.each(characteristic.Fields, function(field) {
		 if(field.BitField) {
            _.each(field.BitField.Bits, function(bitField) {
                _.each(bitField.getEnumerations(), function(enumVal) {
                    if (enumVal.hasRequires()) { %>
        bool <%= enumVal.Requires %>_required = (p_write_val-><%= field.NormalizedName() %>.<%= bitField.NormalizedName() %> == <%= bitField.NormalizedName().toUpperCase() %>_<%= enumVal.NormalizedName().toUpperCase() %>); <%
                    }
                });
	 });}
            _.each(field.getEnumerations(), function(enumVal) {
                if (enumVal.hasRequires()) { %>
        bool <%= enumVal.Requires %>_required = (p_write_val-><%= field.NormalizedName() %>.<%= field.NormalizedName() %> == <%= field.NormalizedName().toUpperCase() %>_<%= enumVal.NormalizedName().toUpperCase() %>); <%
                    }
            });
        });

    var nibble = false;
    _.each(characteristic.Fields, function(field) {
        if(field.hasRequirement()) { %>
    if (<% for(var i=0; i<field.Requirement.length; i++) { if(i>0) { print(' && '); }%><%= field.Requirement[i] %>_required<%}%>)
        { <%
            if (field.isStruct()) {
                if (field.FormatType() === 'nibble') {
                    if (nibble == false) { %>
        nibble = p_data[pos] >> 4;
        uint8_t tmp = (0x0F & p_data[pos++]);
        <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_decode(1, &tmp, &p_write_val-><%= field.NormalizedName() %>);<%
                        nibble = true;
                    } else { %>
        <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_decode(1, &nibble, &p_write_val-><%= field.NormalizedName() %>);
        nibble = 0;<%
                        nibble = false;
                    }
                } else { %>
        pos += <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
                }
            } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8') { %>
        p_write_val-><%= field.NormalizedName() %> = p_data[pos++]; <%
            } else if (field.FormatType() === 'nibble') {
                if (nibble == false) { %>
        nibble = p_data[pos] >> 4;
        p_write_val-><%= field.NormalizedName() %> = 0x0F & p_data[pos++];<%
                    nibble = true;
                } else { %>
        p_write_val-><%= field.NormalizedName() %> = nibble;
        nibble = 0; <%
                    nibble = false;
                }
            } else { %>
        pos += bds_<%= field.FormatType() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
            } %>
        } <%

        } else {
            if (field.isStruct()) {
                if (field.FormatType() === 'nibble') {
                    if (nibble == false) { %>
    nibble = p_data[pos] >> 4;
    uint8_t tmp = (0x0F & p_data[pos++]);
     <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_decode(1, &tmp, &p_write_val-><%= field.NormalizedName() %>);<%
                        nibble = true;
                    } else { %>
     <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_decode(1, &nibble, &p_write_val-><%= field.NormalizedName() %>);
     nibble = 0;<%
                        nibble = false;
                    }
                } else { %>
    pos += <%= characteristic.NormalizedName() %>_<%= field.NormalizedName() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
                }
            } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8') { %>
    p_write_val-><%= field.NormalizedName() %> = p_data[pos++]; <%
            } else if (field.FormatType() === 'nibble') {
                if (nibble == false) { %>
    nibble = p_data[pos] >> 4;
    p_write_val-><%= field.NormalizedName() %> = 0x0F & p_data[pos++];<%
                    nibble = true;
                } else { %>
    p_write_val-><%= field.NormalizedName() %> = nibble;
    nibble = 0; <%
                    nibble = false;
                }
            } else { %>
    pos += bds_<%= field.FormatType() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
            }
         }
    }); %>

    return pos;
} <%
    }
    // ---- BEGIN DESCRIPTOR ENDOCDE/DECODE CODE ---- //



    _.each(characteristic.Descriptors, function(descriptor) {
        if(descriptor.UUID === '2902') { // CCCD, this is already handled in SDK code
            return;
        }
        if(!_.contains(addedDescriptorUuids, descriptor.UUID)) { // Descriptor has not been handled before, keep going
            addedDescriptorUuids.push(descriptor.UUID);
        if (descriptor.isReadSupported() || descriptor.isWriteSupported()) {
        _.each(descriptor.Fields, function(field) {
            if (field.isStruct()) {
    %>
/**@brief Function for encoding <%= field.Name %>.
 *
 * @param[in]   p_<%= field.NormalizedName() %>              <%= field.Name %> structure to be encoded.
 * @param[out]  p_encoded_buffer   Buffer where the encoded data will be written.
 *
 * @return      Size of encoded data.
 */
static uint8_t <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_encode(<%= ShortName %>_<%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_t * p_<%= field.NormalizedName() %>, uint8_t * encoded_buffer)
{
    <%= field.FormatType() %>_t <%= field.NormalizedName() %>;<%
            if (field.BitField && field.BitField.Bits.length > 0) {
                _.each(field.BitField.Bits, function(bitField) { %>
    <%= field.NormalizedName() %> = <%= field.NormalizedName() %> | (p_<%= field.NormalizedName() %>-><%= bitField.NormalizedName() %> << <%= bitField.Index %>); <%
                });
            } else { %>
    <%= field.NormalizedName() %> = p_<%= field.NormalizedName() %>-><%= field.NormalizedName() %>;<%
            }

            if (field.FormatType() === 'uint8' || field.FormatType() === 'int8' || field.FormatType() === 'nibble') { %>
    encoded_buffer[0] = <%= field.NormalizedName() %>;
    return 1; <%
        } else { %>

    return bds_<%= field.FormatType() %>_encode(&<%= field.NormalizedName() %>, &encoded_buffer[0]); <%
        } %>
}

    <%
            }
        });
    %>
/**@brief Function for encoding <%= descriptor.Name %>.
 *
 * @param[in]   p_<%= descriptor.NormalizedName() %>              <%= descriptor.Name %> descriptor structure to be encoded.
 * @param[out]  p_encoded_buffer   Buffer where the encoded data will be written.
 *
 * @return      Size of encoded data.
 */
static uint8_t <%= descriptor.NormalizedName() %>_encode(ble_<%= ShortName %>_<%= descriptor.NormalizedName() %>_t * p_<%= descriptor.NormalizedName() %>, uint8_t * encoded_buffer)
{
    uint8_t len = 0; <%

        if (descriptor.hasNibbles()) { %>
    uint8_t nibble = 0; <%
        }

        _.each(descriptor.Fields, function(field) {
			if(field.BitField) {
                _.each(field.BitField.Bits, function(bitField) {
                    _.each(bitField.getEnumerations(), function(enumVal) {
                        if (enumVal.hasRequires()) { %>
    bool <%= enumVal.Requires %>_required = (p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>.<%= bitField.NormalizedName() %> == <%= bitField.NormalizedName().toUpperCase() %>_<%= enumVal.NormalizedName().toUpperCase() %>); <%
                        }
                    });
		        });
		    }
            _.each(field.getEnumerations(), function(enumVal) {
                if (enumVal.hasRequires()) { %>
    bool <%= enumVal.Requires %>_required = (p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>.<%= field.NormalizedName() %> == <%= field.NormalizedName().toUpperCase() %>_<%= enumVal.NormalizedName().toUpperCase() %>); <%
                    }
            });
        });

        var nibble = false;
        _.each(descriptor.Fields, function(field) {
            if(field.hasRequirement()) { %>
    if (<% for(var i=0; i<field.Requirement.length; i++) { if(i>0) { print(' && '); }%><%= field.Requirement[i] %>_required<%}%>)
    {<%
                if (field.isStruct()) {
                    if (field.FormatType() === 'nibble') {
                        if (nibble == false) { %>
        <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>, &nibble);<%
                            nibble = true;
                        } else { %>
        uint8_t nib;
        <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>, &nib);
        nibble = nibble | nib << 4;
        encoded_buffer[len++] = nibble;
        nibble = 0;<%
                        nibble = false;
                        }
                       } else { %>
        len += <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>, &encoded_buffer[len]); <%
                     }
                } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8') { %>
        encoded_buffer[len++] = p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>;<%
                } else if (field.FormatType() === 'nibble') {
                    if (nibble == false) { %>
        nibble = p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>;<%
                        nibble = true;
                       } else { %>
        nibble = nibble | (p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %> << 4);
        encoded_buffer[len++] = nibble;
        nibble = 0;<%
                        nibble = false;
                       }
                } else { %>
        len += bds_<%= field.FormatType() %>_encode(&p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>, &encoded_buffer[len]); <%
                } %>
    } <%

            } else {
                if (field.isStruct()) {
                    if (field.FormatType() === 'nibble') {
                        if (nibble == false) { %>
    <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>, &nibble);<%
                            nibble = true;
                        } else { %>
    uint8_t nib;
    <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>, &nib);
    nibble = nibble | nib << 4;
    encoded_buffer[len++] = nibble;
    nibble = 0;<%
                            nibble = false;
                        }
                    } else { %>
    len += <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_encode(&p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>, &encoded_buffer[len]); <%
                    }
                } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8') { %>
    encoded_buffer[len++] = p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>;<%
                } else if (field.FormatType() === 'nibble') {
                    if (nibble == false) { %>
    nibble = p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>;<%
                        nibble = true;
                    } else { %>
    nibble = nibble | (p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %> << 4);
    encoded_buffer[len++] = nibble;
    nibble = 0;<%
                        nibble = false;
                    }
                } else { %>
    len += bds_<%= field.FormatType() %>_encode(&p_<%= descriptor.NormalizedName() %>-><%= field.NormalizedName() %>, &encoded_buffer[len]); <%
                }
            }
        });
    %>
    return len;
}
    <%
        }

        if (descriptor.isWriteSupported()) {
         _.each(descriptor.Fields, function(field) {
                 if (field.isStruct()) {
         %>
 /**@brief Function for decoding <%= field.Name %>.
 *
 * @param[in]   data_len              Length of the field to be decoded.
 * @param[in]   p_data                Buffer where the encoded data is stored.
 * @param[out]  p_write_val           Decoded data.
 *
 * @return      Length of the decoded field.
 */
 static uint8_t <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_decode(uint8_t data_len, uint8_t * p_data, <%= ShortName %>_<%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_t * p_write_val)
 {
    uint8_t pos = 0; <%

            if (field.BitField && field.BitField.Bits.length > 0) {
                _.each(field.BitField.Bits, function(bitField) { %>
    p_write_val-><%= bitField.NormalizedName() %> = (enum_<%= field.NormalizedName() %>_<%= bitField.NormalizedName() %>_t)(p_data[pos] & (0x01 <<  <%= bitField.Index %>));<%
                }); %>
    pos += sizeof(<%= field.FormatType() %>_t);<%
            } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8' || field.FormatType() === 'nibble') { %>
    p_write_val-><%= field.NormalizedName() %> = <% if (field.isStruct()) { %>(enum_<%= field.NormalizedName() %>_t)<% }%>p_data[pos++]; <%
            } else { %>
    pos += bds_<%= field.FormatType() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
            } %>
    return pos;
 }

         <%
                 }
             });
        %>
/**@brief Function for decoding <%= descriptor.Name %>.
 *
 * @param[in]   data_len              Length of the field to be decoded.
 * @param[in]   p_data                Buffer where the encoded data is stored.
 * @param[out]  p_write_val           Decoded data.
 *
 * @return      Length of the decoded field.
 */
static uint8_t <%= descriptor.NormalizedName() %>_decode(uint8_t data_len, uint8_t * p_data, ble_<%= ShortName %>_<%= descriptor.NormalizedName() %>_t * p_write_val)
{
    uint8_t pos = 0;<%

        if (descriptor.hasNibbles()) { %>
    uint8_t nibble = 0; <%
        }
         _.each(descriptor.Fields, function(field) {
			 if(field.BitField) {
                _.each(field.BitField.Bits, function(bitField) {
                    _.each(bitField.getEnumerations(), function(enumVal) {
                        if (enumVal.hasRequires()) { %>
    bool <%= enumVal.Requires %>_required = (p_write_val-><%= field.NormalizedName() %>.<%= bitField.NormalizedName() %> == <%= bitField.NormalizedName().toUpperCase() %>_<%= enumVal.NormalizedName().toUpperCase() %>); <%
                        }
                    });
		 }); }
                _.each(field.getEnumerations(), function(enumVal) {
                    if (enumVal.hasRequires()) { %>
    bool <%= enumVal.Requires %>_required = (p_write_val-><%= field.NormalizedName() %>.<%= field.NormalizedName() %> == <%= field.NormalizedName().toUpperCase() %>_<%= enumVal.NormalizedName().toUpperCase() %>); <%
                        }
                });
            });

        var nibble = false;
        _.each(descriptor.Fields, function(field) {
            if(field.hasRequirement()) { %>
    if (<% for(var i=0; i<field.Requirement.length; i++) { if(i>0) { print(' && '); }%><%= field.Requirement[i] %>_required<%}%>)
    { <%
                if (field.isStruct()) {
                    if (field.FormatType() === 'nibble') {
                        if (nibble == false) { %>
        nibble = p_data[pos] >> 4;
        uint8_t tmp = (0x0F & p_data[pos++]);
        <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_decode(1, &tmp, &p_write_val-><%= field.NormalizedName() %>);<%
                            nibble = true;
                        } else { %>
        <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_decode(1, &nibble, &p_write_val-><%= field.NormalizedName() %>);
        nibble = 0;<%
                            nibble = false;
                        }
                    } else { %>
        pos += <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
                    }
                } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8') { %>
        p_write_val-><%= field.NormalizedName() %> = p_data[pos++]; <%
                } else if (field.FormatType() === 'nibble') {
                    if (nibble == false) { %>
        nibble = p_data[pos] >> 4;
        p_write_val-><%= field.NormalizedName() %> = 0x0F & p_data[pos++];<%
                        nibble = true;
                    } else { %>
        p_write_val-><%= field.NormalizedName() %> = nibble;
        nibble = 0; <%
                        nibble = false;
                    }
                } else { %>
        pos += bds_<%= field.FormatType() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
                } %>
    } <%

            } else {
                if (field.isStruct()) {
                    if (field.FormatType() === 'nibble') {
                        if (nibble == false) { %>
    nibble = p_data[pos] >> 4;
    uint8_t tmp = (0x0F & p_data[pos++]);
    <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_decode(1, &tmp, &p_write_val-><%= field.NormalizedName() %>);<%
                            nibble = true;
                        } else { %>
    <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_decode(1, &nibble, &p_write_val-><%= field.NormalizedName() %>);
    nibble = 0;<%
                            nibble = false;
                        }
                    } else { %>
    pos += <%= descriptor.NormalizedName() %>_<%= field.NormalizedName() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
                    }
                } else if (field.FormatType() === 'uint8' || field.FormatType() === 'int8') { %>
    p_write_val-><%= field.NormalizedName() %> = p_data[pos++]; <%
                } else if (field.FormatType() === 'nibble') {
                    if (nibble == false) { %>
    nibble = p_data[pos] >> 4;
    p_write_val-><%= field.NormalizedName() %> = 0x0F & p_data[pos++];<%
                        nibble = true;
                    } else { %>
    p_write_val-><%= field.NormalizedName() %> = nibble;
    nibble = 0; <%
                        nibble = false;
                    }
                } else { %>
    pos += bds_<%= field.FormatType() %>_decode((data_len-pos), &p_data[pos], &p_write_val-><%= field.NormalizedName() %>); <%
                }
             }
        }); %>

    return pos;
} <%
        }
        }
    });

    // ---- END DESCRIPTOR ENDOCDE/DECODE CODE ---- //

});
%>

/**@brief Function for handling the Connect event.
 *
 * @param[in]   p_<%= ShortName %>       <%= Name %> Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_connect(ble_<%= ShortName %>_t * p_<%= ShortName %>, ble_evt_t * p_ble_evt)
{
    p_<%= ShortName %>->conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
}

/**@brief Function for handling the Disconnect event.
 *
 * @param[in]   p_<%= ShortName %>       <%= Name %> Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_disconnect(ble_<%= ShortName %>_t * p_<%= ShortName %>, ble_evt_t * p_ble_evt)
{
    UNUSED_PARAMETER(p_ble_evt);
    p_<%= ShortName %>->conn_handle = BLE_CONN_HANDLE_INVALID;
}

/**@brief Function for handling the Write event.
 *
 * @param[in]   p_<%= ShortName %>       <%= Name %> Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
static void on_write(ble_<%= ShortName %>_t * p_<%= ShortName %>, ble_gatts_evt_write_t * p_ble_evt)
{
    <%
    _.each(Characteristics, function(characteristic) {
        if (characteristic.isNotifySupported() || characteristic.isIndicateSupported()) { %>
    if(p_ble_evt->handle == p_<%= ShortName %>-><%= characteristic.NormalizedName() %>_handles.cccd_handle)
    {
        if(p_<%= ShortName %>->evt_handler != NULL)
        {
            ble_<%= ShortName %>_evt_t evt;
            evt.evt_type = BLE_<%= ShortName.toUpperCase() %>_<%= characteristic.NormalizedName().toUpperCase() %>_EVT_CCCD_WRITE;
            bds_uint16_decode(p_ble_evt->len, p_ble_evt->data, &evt.params.cccd_value);
            p_<%= ShortName %>->evt_handler(p_<%= ShortName %>, &evt);
        }
    } <%
        }

        if (characteristic.isWriteSupported() || characteristic.isWriteWithoutResponseSupported() || characteristic.isReliableWriteSupported()) { %>
    if(p_ble_evt->handle == p_<%= ShortName %>-><%= characteristic.NormalizedName() %>_handles.value_handle)
    {
        if(p_<%= ShortName %>->evt_handler != NULL)
        {
            ble_<%= ShortName %>_evt_t evt;
            evt.evt_type = BLE_<%= ShortName.toUpperCase() %>_<%= characteristic.NormalizedName().toUpperCase() %>_EVT_WRITE;
            <%= characteristic.NormalizedName() %>_decode(p_ble_evt->len, p_ble_evt->data, &evt.params.<%= characteristic.NormalizedName() %>);
            p_<%= ShortName %>->evt_handler(p_<%= ShortName %>, &evt);
        }
    }<%
        }
    });%>
}

/**@brief Authorize WRITE request event handler.
 *
 * @details Handles WRITE events from the BLE stack.
 *
 * @param[in]   p_sc_ctrlpt  SC Ctrlpt structure.
 * @param[in]   p_gatts_evt  GATTS Event received from the BLE stack.
 *
 */
static void on_rw_authorize_request(ble_<%= ShortName %>_t * p_<%= ShortName %>, ble_gatts_evt_t * p_gatts_evt)
{
    ble_gatts_evt_rw_authorize_request_t * p_auth_req = &p_gatts_evt->params.authorize_request;
    if (p_auth_req->type == BLE_GATTS_AUTHORIZE_TYPE_WRITE)
    {
        if (   (p_gatts_evt->params.authorize_request.request.write.op
                != BLE_GATTS_OP_PREP_WRITE_REQ)
            && (p_gatts_evt->params.authorize_request.request.write.op
                != BLE_GATTS_OP_EXEC_WRITE_REQ_NOW)
            && (p_gatts_evt->params.authorize_request.request.write.op
                != BLE_GATTS_OP_EXEC_WRITE_REQ_CANCEL)
           )
        {
        <%
            _.each(Characteristics, function(characteristic) {
                if (characteristic.isWriteSupported())
                { %>
            if (p_auth_req->request.write.handle == p_<%= ShortName %>-><%= characteristic.NormalizedName() %>_handles.value_handle)
            {
                on_write(p_<%= ShortName %>, &p_auth_req->request.write);
            }<%
                 }
            });%>
        }
    }
}

/**@brief Function for handling BLE events.
 *
 * @param[in]   p_<%= ShortName %>       <%= Name %> Service structure.
 * @param[in]   p_ble_evt   Event received from the BLE stack.
 */
void ble_<%= ShortName %>_on_ble_evt(ble_<%= ShortName %>_t * p_<%= ShortName %>, ble_evt_t * p_ble_evt)
{
    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            on_connect(p_<%= ShortName %>, p_ble_evt);
            break;
        case BLE_GAP_EVT_DISCONNECTED:
            on_disconnect(p_<%= ShortName %>, p_ble_evt);
            break;
        case BLE_GATTS_EVT_WRITE:
            on_write(p_<%= ShortName %>, &p_ble_evt->evt.gatts_evt.params.write);
            break;
         case BLE_GATTS_EVT_RW_AUTHORIZE_REQUEST:
            on_rw_authorize_request(p_<%= ShortName %>, &p_ble_evt->evt.gatts_evt);
            break;
        default:
            //No implementation needed.
            break;
    }
}

/**@brief Function for initializing the <%= Name %>. */
uint32_t ble_<%= ShortName %>_init(ble_<%= ShortName %>_t * p_<%= ShortName %>, const ble_<%= ShortName %>_init_t * p_<%= ShortName %>_init)
{
    uint32_t err_code;
    ble_uuid_t ble_uuid;

    // Initialize service structure
    p_<%= ShortName %>->evt_handler = p_<%= ShortName %>_init->evt_handler;
    p_<%= ShortName %>->conn_handle = BLE_CONN_HANDLE_INVALID;
    <%
        if (!_.isNull(FullUuid)) { %>
    // Add a custom base UUID.
    ble_uuid128_t bds_base_uuid = {{<%= FullUuid.UuidBase %>}};
    uint8_t       uuid_type;
    err_code = sd_ble_uuid_vs_add(&bds_base_uuid, &uuid_type);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
    ble_uuid.type = uuid_type;
    ble_uuid.uuid = <%= FullUuid.Uuid %>;<%
        } else { %>
    BLE_UUID_BLE_ASSIGN(ble_uuid, 0x<%= UUID %>);<%
        }%>
        
    // Add service
    err_code = sd_ble_gatts_service_add(BLE_GATTS_SRVC_TYPE_PRIMARY, &ble_uuid, &p_<%= ShortName %>->service_handle);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    } <%

    _.each(Characteristics, function(characteristic) {
        if (characteristic.isReadSupported() || characteristic.isNotifySupported() || characteristic.isIndicateSupported() ||
            characteristic.isWriteSupported() || characteristic.isWriteWithoutResponseSupported() || characteristic.isReliableWriteSupported()) { %>

    // Add <%= characteristic.Name %> characteristic
    <%= characteristic.StructName() %> <%= characteristic.NormalizedName() %>_initial_value = p_<%= ShortName %>_init->ble_<%= ShortName %>_<%= characteristic.NormalizedName() %>_initial_value; <%
    if (characteristic.isReadSupported() && !characteristic.isReadMandatory()) { %>
    p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_read_supported = p_<%= ShortName %>_init->is_<%= characteristic.NormalizedName() %>_read_supported;<%
    }
    if (characteristic.isNotifySupported() && !characteristic.isNotifyMandatory()) { %>
    p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_notify_supported = p_<%= ShortName %>_init->is_<%= characteristic.NormalizedName() %>_notify_supported;<%
    }
    if (characteristic.isIndicateSupported() && !characteristic.isIndicateMandatory()) { %>
    p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_indicate_supported = p_<%= ShortName %>_init->is_<%= characteristic.NormalizedName() %>_indicate_supported;<%
    }
    if (characteristic.isWriteSupported() && !characteristic.isWriteMandatory()) { %>
    p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_write_supported = p_<%= ShortName %>_init->is_<%= characteristic.NormalizedName() %>_write_supported;<%
    }
    if (characteristic.isWriteWithoutResponseSupported() && !characteristic.isWriteWithoutResponseMandatory()) { %>
    p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_write_wo_resp_supported = p_<%= ShortName %>_init->is_<%= characteristic.NormalizedName() %>_write_wo_resp_supported;<%
    }
    if (characteristic.isReliableWriteSupported() && !characteristic.isReliableWriteMandatory()) { %>
     p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_reliable_write_supported = p_<%= ShortName %>_init->is_<%= characteristic.NormalizedName() %>_reliable_write_supported;<%
     } %>

    uint8_t <%= characteristic.NormalizedName() %>_encoded_value[MAX_<%= characteristic.NormalizedName().toUpperCase() %>_LEN];
    ble_add_char_params_t add_<%= characteristic.NormalizedName() %>_params;
    memset(&add_<%= characteristic.NormalizedName() %>_params, 0, sizeof(add_<%= characteristic.NormalizedName() %>_params));
    <%
    if(_.isNull(characteristic.getFullUuid())) { %>
    add_<%= characteristic.NormalizedName() %>_params.uuid                = 0x<%= characteristic.UUID %>; <%
    } else { %>
    add_<%= characteristic.NormalizedName() %>_params.uuid                = <%= characteristic.getFullUuid().Uuid %>;
    add_<%= characteristic.NormalizedName() %>_params.uuid_type           = ble_uuid.type; <%
    } %>
    add_<%= characteristic.NormalizedName() %>_params.max_len             = MAX_<%= characteristic.NormalizedName().toUpperCase() %>_LEN;
    add_<%= characteristic.NormalizedName() %>_params.init_len            = <%= characteristic.NormalizedName() %>_encode(&<%= characteristic.NormalizedName() %>_initial_value, <%= characteristic.NormalizedName() %>_encoded_value);
    add_<%= characteristic.NormalizedName() %>_params.p_init_value        = <%= characteristic.NormalizedName() %>_encoded_value; <%
    if (characteristic.isNotifyMandatory()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.notify   = 1; <%
    } else if (characteristic.isNotifySupported()){ %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.notify   = p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_notify_supported; <%
    }

    if (characteristic.isIndicateMandatory()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.indicate = 1; <%
    } else if (characteristic.isIndicateSupported()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.indicate = p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_indicate_supported; <%
    }

    if (characteristic.isReadMandatory()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.read     = 1; <%
    } else if (characteristic.isReadSupported()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.read = p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_read_supported; <%
    }

    if (characteristic.isReadSupported()) {     %>
    add_<%= characteristic.NormalizedName() %>_params.read_access         = SEC_OPEN; <%
    }
    
    if (characteristic.isWriteMandatory()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.write    = 1; <%
    } else if (characteristic.isWriteSupported()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.write = p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_write_supported; <%
    }

    if (characteristic.isWriteWithoutResponseMandatory()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.write_wo_resp    = 1; <%
    } else if (characteristic.isWriteWithoutResponseSupported()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.write_wo_resp = p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_write_wo_resp_supported; <%
    }

    if (characteristic.isReliableWriteMandatory()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.reliable_write    = 1; <%
    } else if (characteristic.isReliableWriteSupported()) { %>
    add_<%= characteristic.NormalizedName() %>_params.char_props.reliable_write = p_<%= ShortName %>->is_<%= characteristic.NormalizedName() %>_reliable_write_supported; <%
    }

    if (characteristic.isWriteSupported() || characteristic.isWriteWithoutResponseSupported() || characteristic.isReliableWriteSupported()) {     %>
    add_<%= characteristic.NormalizedName() %>_params.write_access        = SEC_OPEN; <%
    }

    if (characteristic.isNotifySupported() || characteristic.isIndicateSupported()) { %>
    add_<%= characteristic.NormalizedName() %>_params.cccd_write_access   = SEC_OPEN;<%
    }
    %>
    // 1 for variable length and 0 for fixed length.
    add_<%= characteristic.NormalizedName() %>_params.is_var_len          = 1; <%

    if(!characteristic.HasDescriptorAddSpecial) { // We need to add the characteristic before adding "regular" descriptors
    %>

    err_code = characteristic_add(p_<%= ShortName %>->service_handle, &add_<%= characteristic.NormalizedName() %>_params, &(p_<%= ShortName %>-><%= characteristic.NormalizedName()%>_handles));
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    } <%
    }

    addedDescriptorUuids.length = 0;
    _.each(characteristic.Descriptors, function(descriptor) {
         if(descriptor.UUID === '2902') { // CCCD, this is already handled in SDK code
             return;
         } %>

    // Add <%= descriptor.Name %> descriptor <%
        if(!_.contains(addedDescriptorUuids, descriptor.UUID)) { // Descriptor has not been handled before, keep going
            addedDescriptorUuids.push(descriptor.UUID);
        if(!_.isUndefined(descriptor.DescriptorAddSpecial)) { // There should only be ONE such item per characteristic!
            if(descriptor.UUID === '2901') { // Characteristic User Description %>
    ble_add_char_user_desc_t <%= descriptor.NormalizedName() %>;
    uint8_t <%= descriptor.NormalizedName() %>_val[]  = {'b', 'd', 's', '\0'};

    memset(&<%= descriptor.NormalizedName() %>, 0, sizeof(<%= descriptor.NormalizedName() %>));
    <%= descriptor.NormalizedName() %>.read_access      = SEC_OPEN;
    <%= descriptor.NormalizedName() %>.write_access     = SEC_NO_ACCESS;
    <%= descriptor.NormalizedName() %>.max_size          = MAX_<%= descriptor.NormalizedName().toUpperCase() %>_LEN;
    <%= descriptor.NormalizedName() %>.size              = MAX_<%= descriptor.NormalizedName().toUpperCase() %>_LEN;
    <%= descriptor.NormalizedName() %>.is_value_user     = false;
    <%= descriptor.NormalizedName() %>.p_char_user_desc  = <%= descriptor.NormalizedName() %>_val;
    add_<%= characteristic.NormalizedName() %>_params.p_user_descr = &<%= descriptor.NormalizedName() %>; <%
            }
            else if(descriptor.UUID === '2904') { // Characteristic Presentation Format %>
    ble_gatts_char_pf_t <%= descriptor.NormalizedName() %>;

    memset(&<%= descriptor.NormalizedName() %>, 0, sizeof(<%= descriptor.NormalizedName() %>));

    <%= descriptor.NormalizedName() %>.format     = BLE_GATT_CPF_FORMAT_UINT64;   //See BLE_GATT_CPF_FORMATS in ble_gatt.h file for the list of possible values
    <%= descriptor.NormalizedName() %>.desc       = BLE_GATT_CPF_NAMESPACE_BTSIG; //See BLE_GATT_CPF_NAMESPACES in ble_gatt.h file for the list of possible values
    <%= descriptor.NormalizedName() %>.exponent   = -2;                           //Exponent for integer data types.
    <%= descriptor.NormalizedName() %>.name_space = BLE_GATT_CPF_NAMESPACE_BTSIG; //See BLE_GATT_CPF_NAMESPACES in ble_gatt.h file for the list of possible values
    <%= descriptor.NormalizedName() %>.unit       = 0x2702;                       //corresponds to mass (kilogram), see https://developer.bluetooth.org/gatt/units/Pages/default.aspx for the list of units.
    add_<%= characteristic.NormalizedName() %>_params.p_presentation_format = &<%= descriptor.NormalizedName() %>; <%
            } %>

    err_code = characteristic_add(p_<%= ShortName %>->service_handle, &add_<%= characteristic.NormalizedName() %>_params, &(p_<%= ShortName %>-><%= characteristic.NormalizedName()%>_handles));
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    } <%
        }
        else {
            // Add regular descriptions, and "special" ones from #2->
    %>
    ble_add_descr_params_t   <%= descriptor.NormalizedName() %>;
    uint16_t                 descr_handle;
    uint8_t                  descr_val[MAX_<%= descriptor.NormalizedName().toUpperCase() %>_LEN];

    memset(&<%= descriptor.NormalizedName() %>, 0, sizeof(<%= descriptor.NormalizedName() %>));

    <%= descriptor.NormalizedName() %>.uuid             = BLE_UUID_REPORT_REF_DESCR;
    <%= descriptor.NormalizedName() %>.read_access      = SEC_OPEN;
    <%= descriptor.NormalizedName() %>.write_access     = SEC_NO_ACCESS;
    <%= descriptor.NormalizedName() %>.init_len         = MAX_<%= descriptor.NormalizedName().toUpperCase() %>_LEN;
    <%= descriptor.NormalizedName() %>.max_len          = MAX_<%= descriptor.NormalizedName().toUpperCase() %>_LEN;
    <%= descriptor.NormalizedName() %>.p_value          = descr_val;

    err_code = descriptor_add(BLE_GATT_HANDLE_INVALID, &<%= descriptor.NormalizedName() %>, &descr_handle);
    if (err_code != NRF_SUCCESS)
    {
        return err_code;
    }
    <%
        }
        }
    });
    }
    });%>

    return NRF_SUCCESS;
}
<%

_.each(Characteristics, function(characteristic) {
    if (characteristic.isReadSupported()) {
%>
/**@brief Function for setting the <%= characteristic.Name %>. */
uint32_t ble_<%= ShortName %>_<%= characteristic.NormalizedName()%>_set(ble_<%= ShortName %>_t * p_<%= ShortName %>, ble_<%= ShortName %>_<%= characteristic.NormalizedName()%>_t * p_<%= characteristic.NormalizedName() %>)
{
    ble_gatts_value_t gatts_value;
    uint8_t encoded_value[MAX_<%= characteristic.NormalizedName().toUpperCase() %>_LEN];

    // Initialize value struct.
    memset(&gatts_value, 0, sizeof(gatts_value));

    gatts_value.len     = <%= characteristic.NormalizedName() %>_encode(p_<%= characteristic.NormalizedName() %>, encoded_value);
    gatts_value.offset  = 0;
    gatts_value.p_value = encoded_value;

    return sd_ble_gatts_value_set(p_<%= ShortName %>->conn_handle, p_<%= ShortName %>-><%= characteristic.NormalizedName() %>_handles.value_handle, &gatts_value);
}
<%
    }
    addedDescriptorUuids.length = 0;
    _.each(characteristic.Descriptors, function(descriptor) {
        if(descriptor.UUID === '2902') { // CCCD, this is already handled in SDK code
            return;
        }
        if(!_.contains(addedDescriptorUuids, descriptor.UUID)) { // Descriptor has not been handled before, keep going
            if(characteristic.isReadSupported()) { %>
/**@brief Function for setting the <%= descriptor.Name %>. */
uint32_t ble_<%= ShortName %>_<%= descriptor.NormalizedName()%>_set(ble_<%= ShortName %>_t * p_<%= ShortName %>, ble_<%= ShortName %>_<%= descriptor.NormalizedName()%>_t * p_<%= descriptor.NormalizedName() %>)
{
    ble_gatts_value_t gatts_value;
    uint8_t encoded_value[MAX_<%= descriptor.NormalizedName().toUpperCase() %>_LEN];

    // Initialize value struct.
    memset(&gatts_value, 0, sizeof(gatts_value));

    gatts_value.len     = <%= descriptor.NormalizedName() %>_encode(p_<%= descriptor.NormalizedName() %>, encoded_value);
    gatts_value.offset  = 0;
    gatts_value.p_value = encoded_value;

    return sd_ble_gatts_value_set(p_<%= ShortName %>->conn_handle, p_<%= ShortName %>-><%= descriptor.NormalizedName() %>_handles.value_handle, &gatts_value);
} <%
            }
            if(descriptor.isWriteSupported()) {
        // TODO: What about descriptor.Write?
            }
        }
    });

    _.each(characteristic.SendFunction(), function(sendFunc) { %>
/**@brief Function for sending the <%= characteristic.Name %>. */
uint32_t ble_<%= ShortName %>_<%= characteristic.NormalizedName()%>_<%= sendFunc.Postfix %>(ble_<%= ShortName %>_t * p_<%= ShortName %>, ble_<%= ShortName %>_<%= characteristic.NormalizedName()%>_t * p_<%= characteristic.NormalizedName() %>)
{
    uint32_t err_code = NRF_SUCCESS;

    if (p_<%= ShortName %>->conn_handle != BLE_CONN_HANDLE_INVALID)
    {
        ble_gatts_hvx_params_t hvx_params;
        uint8_t encoded_value[MAX_<%= characteristic.NormalizedName().toUpperCase() %>_LEN];
        uint16_t hvx_len;

        // Initialize value struct.
        memset(&hvx_params, 0, sizeof(hvx_params));

        hvx_len           = <%= characteristic.NormalizedName() %>_encode(p_<%= characteristic.NormalizedName() %>, encoded_value);
        hvx_params.handle = p_<%= ShortName %>-><%= characteristic.NormalizedName() %>_handles.value_handle;
        hvx_params.type   = <%= sendFunc.Type %>;
        hvx_params.p_len  = &hvx_len;
        hvx_params.offset = 0;
        hvx_params.p_data = encoded_value;

        err_code = sd_ble_gatts_hvx(p_<%= ShortName %>->conn_handle, &hvx_params);
    }
    else
    {
        err_code = NRF_ERROR_INVALID_STATE;
    }

    return err_code;
}
<%
    });
});
%>
